/*
 *      Copyright (C) 2009 Antti Ajanki <antti.ajanki@iki.fi>
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with RTMPDump; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *  http://www.gnu.org/copyleft/gpl.html
 *
 */

/*
 * AkamaiRTMP implements the extra functionality that is needed for
 * connecting to Akamai network. The difference to plain rtmpdump is
 * that before calling playStream the client has to call
 * FCSubscribe("playpath?auth=...") and wait for onFCSubscribe
 * invocation from the server.
 */

#include <string.h>

#include "rtmppacket.h"
#include "akamai.h"

using namespace AKAMAI;
using namespace RTMP_LIB;

void AkamaiRTMP::ExecuteInvokedMethod(const RTMP_LIB::AMFObject& obj) {
  std::string method = obj.GetProperty(0).GetString();
  Log(LOGDEBUG, "%s, server invoking <%s>", __FUNCTION__, method.c_str());

  if ((method == "_result") && (m_methodCalls[0] == "createStream")) {
    m_methodCalls.erase(m_methodCalls.begin());

    m_stream_id = (int)obj.GetProperty(3).GetNumber();

    Log(LOGDEBUG, "%s, received result for method call <createStream>", __FUNCTION__);

    SendFCSubscribe();

  } else if (method == "onFCSubscribe") {
    std::string code  = obj.GetProperty(3).GetObject().GetProperty("code").GetString();

    Log(LOGDEBUG, "%s, onFCSubscribe: %s", __FUNCTION__, code.c_str() );

    if (code == "NetStream.Play.Start")
      SendPlay();
    else
      Close();

  } else {
    CRTMP::ExecuteInvokedMethod(obj);
  }

}

bool AkamaiRTMP::SendFCSubscribe() {
  RTMPPacket packet;
  packet.m_nChannel = 0x03; // control channel
  packet.m_headerType = RTMP_PACKET_SIZE_LARGE;
  packet.m_packetType = 0x14; // INVOKE
  packet.m_nInfoField1 = GetTime();

  packet.AllocPacket(30+strlen(Link.playpath));
  char *enc = packet.m_body;
  enc += EncodeString(enc, "FCSubscribe");
  enc += EncodeNumber(enc, 0.0);
  *enc = 0x05; // NULL
  enc++;
  enc += EncodeString(enc, Link.playpath);
  
  Log(LOGDEBUG, "%s, sending FCSubscribe: %s", __FUNCTION__, Link.playpath);

  packet.m_nBodySize = enc - packet.m_body;

  return SendRTMP(packet);
}
