#ifndef __YLE_H__
#define __YLE_H__
/*
 *      Copyright (C) 2005-2008 Team XBMC
 *      http://www.xbmc.org
 *	Copyright (C) 2008-2009 Andrej Stepanchuk
 *      Copyright (C) 2009 Antti Ajanki
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with RTMPDump; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *  http://www.gnu.org/copyleft/gpl.html
 *
 */

#include <string.h>

#include "rtmp.h"

#define YLERTMPDUMP_VERSION	"1.1.7"

namespace YLE_AREENA
{

char *FetchLiveURL(const char *userAgent);
char *GetHTTP(const char *url, const char *userAgent);
char *LoadAreenaParams(const char *url, const char *userAgent);
void ParseAreenaParams(const char *areenaParams, 
                       char **clipID, 
                       char **liveStreamID, 
                       char **radioID,
                       char **language, 
                       char **hostname, 
                       int *port, 
                       int *protocol, 
                       char **app);

class AreenaLink {
public:
  AreenaLink();
  ~AreenaLink();

  void Reset();

  char *clipID;
  char *liveStreamID;
  char *radioID;
  char *language;
  bool tvFeeRequired;
  bool tvFeeActivated;
  bool locatedInBroadcastTerritory;
  bool isAkamaiStream;
};

class YLERTMP : public RTMP_LIB::CRTMP {
public:
  YLERTMP();
  virtual ~YLERTMP();

  void SetupStream(
      int protocol, 
      const char *hostname, 
      unsigned int port, 
      const char *playpath, 
      const char *tcUrl, 
      const char *swfUrl, 
      const char *pageUrl, 
      const char *app, 
      const char *clipID,
      const char *liveStreamID,
      const char *radioID,
      const char *language,
      const char *auth,
      const char *swfSHA256Hash,
      uint32_t swfSize,
      const char *flashVer, 
      double dTime,
      bool bLiveStream,
      long int timeout=300);

  bool Connect();

  RTMP_LIB::CRTMP *GetRedirection();

  std::string GetSuggestedFileName();
  std::string SaneFileName(const std::string& filename);

  bool TVFeeRequiredAndMissing();
  
protected:
  std::string streamTitle;
  char *fileextension;

  // stream-related variables from the server
  AreenaLink areenaStream;
  RTMP_LIB::LinkData akamaiStream;

  void ExecuteInvokedMethod(const RTMP_LIB::AMFObject& obj);
  bool ExecuteAuthenticationDetails(const RTMP_LIB::AMFObject& properties);
  bool ExecuteRandomNumberAuthenticated();
  bool ExecuteRPCResult(const RTMP_LIB::AMFObject& obj);

  char *UrlToPlaypath(const char *url);

  void LoadAkamaiXML(const char *url);
  void ParseAkamaiXML(const char *xml, const char *xmlurl);

  bool RequestData(const std::string& request, const std::string& id);
  bool ClipsInfo();
  bool SessionAuthenticate();
  bool RelatedClips();
  bool FeaturedClips();
  bool ChannelsList();
  bool CategoriesList();
};

}

#endif
